# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

import logging
from sweetrpg_model_core.model.base import Model


class Contribution(Model):
    """A model object representing contributions to an RPG volume contribution."""

    def __init__(self, *args, **kwargs):
        """Creates a new Contribution object.

        :key str person_id: The identifier of the contributor.
        :key list roles: A list of role values indicating the contributions.
        """
        logging.debug("args: %s, kwargs: %s", args, kwargs)
        print(kwargs)  # TODO: remove

        super().__init__(*args, **kwargs)

        self.person_id = kwargs.get("person_id")
        self.volume_id = kwargs.get("volume_id")
        self.roles = kwargs.get("roles")



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow_jsonapi import fields
from marshmallow_jsonapi.flask import Relationship
from sweetrpg_api_core.schema.base import BaseAPISchema
from sweetrpg_catalog_objects.model.license import License


class LicenseAPISchema(BaseAPISchema):
    model_class = License

    class Meta:
        type_ = "license"
        self_view = "license_detail"
        self_view_kwargs = {"id": "<id>"}
        self_view_many = "license_list"

    title = fields.String(required=True)  # , load_only=True)
    short_title = fields.String(required=True)  # , load_only=True)
    version = fields.String(required=True)  # , load_only=True)
    deed = fields.String()  # , load_only=True)
    legal_code = fields.String()  # , load_only=True)
    url = fields.String()  # , load_only=True)
    status = fields.String(required=True)  # , load_only=True)
    availability = fields.String(required=True)  # , load_only=True)
    release_date = fields.Date()  # , load_only=True)
    revocation_date = fields.Date()  # , load_only=True)
    notes = fields.String()  # , load_only=True)
    properties = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    tags = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    volume_ids = fields.List(fields.String(required=True))  # , load_only=True)



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow import fields
from sweetrpg_catalog_objects.model.embedded.property import Property
from sweetrpg_model_core.schema.base import BaseEmbeddedSchema


class PropertySchema(BaseEmbeddedSchema):
    model_class = Property

    name = fields.String(required=True)  # , load_only=True)
    value = fields.String(required=True)  # , load_only=True)
    kind = fields.String(allow_none=True)  # , load_only=True)



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
Volume document for MongoDB.
"""

from datetime import datetime
from mongoengine import Document, fields
from sweetrpg_catalog_objects.db.embedded.property.document import PropertyDocument
from sweetrpg_catalog_objects.db.embedded.tag.document import TagDocument


class VolumeDocument(Document):
    """
    A mapping object to convert MongoDB data to a Volume object.
    """

    meta = {
        "indexes": [
            {"name": "volume_title", "fields": ["title"]},
        ],
        "db_alias": "default",
        "collection": "volumes",
        "strict": False,
    }

    source_id = fields.StringField()

    # basic properties
    title = fields.StringField(required=True)
    description = fields.StringField(required=True)
    properties = fields.ListField(fields.EmbeddedDocumentField(PropertyDocument))
    tags = fields.ListField(fields.EmbeddedDocumentField(TagDocument))
    systems = fields.ListField(fields.ReferenceField("SystemDocument"))
    publishers = fields.ListField(fields.ReferenceField("PublisherDocument"))
    studios = fields.ListField(fields.ReferenceField("StudioDocument"))
    licenses = fields.ListField(fields.ReferenceField("LicenseDocument"))

    # audit properties
    created_at = fields.DateTimeField(default=datetime.utcnow, required=True)
    created_by = fields.StringField(default="system", required=True)
    updated_at = fields.DateTimeField(default=datetime.utcnow, required=True)
    updated_by = fields.StringField(default="system", required=True)
    deleted_at = fields.DateTimeField(null=True)
    deleted_by = fields.StringField(default="system", required=True)



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

import logging
from sweetrpg_model_core.model.base import Model


class System(Model):
    """A model object representing an RPG game system."""

    def __init__(self, *args, **kwargs):
        """Creates a new System object.

        :key str name: The name of the system.
        """
        logging.debug("args: %s, kwargs: %s", args, kwargs)

        super().__init__(*args, **kwargs)

        self.game_system = kwargs.get("game_system")
        self.name = kwargs.get("name")
        self.edition = kwargs.get("edition")
        self.tags = kwargs.get("tags")



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow import fields
from sweetrpg_catalog_objects.model.publisher import Publisher
from sweetrpg_model_core.schema.base import BaseSchema


class PublisherSchema(BaseSchema):
    model_class = Publisher

    name = fields.String(required=True)  # , load_only=True)
    address = fields.String()
    website = fields.String()
    notes = fields.String()
    tags = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    properties = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from datetime import datetime
from mongoengine import EmbeddedDocument, fields


class TagDocument(EmbeddedDocument):
    """A mapping object to convert MongoDB data to a Tag object."""

    # basic properties
    name = fields.StringField(min_length=1, max_length=50, required=True)
    value = fields.StringField(max_length=50)



# -*- coding: utf-8 -*-
"""
Metadata.
"""

__title__ = "sweetrpg-catalog-objects"
__description__ = "Model objects for SweetRPG Catalog"
__url__ = "https://sweetrpg.com"
__version__ = "0.0.125"
__build__ = 0x000001
__author__ = "Paul Schifferer"
__author_email__ = "dm@sweetrpg.com"
__license__ = "MIT"
__copyright__ = "Copyright 2023 SweetRPG"
__cake__ = "\u2728 \U0001f370 \u2728"



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from datetime import datetime
from mongoengine import Document, fields
from sweetrpg_catalog_objects.db.embedded.tag.document import TagDocument


class ReviewDocument(Document):
    """A mapping object to convert MongoDB data to a Review object."""

    meta = {
        "indexes": [
            {"name": "review_title", "fields": ["title"]},
        ],
        "db_alias": "default",
        "collection": "reviews",
        "strict": False,
    }

    source_id = fields.StringField()

    # basic properties
    title = fields.StringField(required=True)
    body = fields.StringField(required=True)
    language = fields.StringField(required=True, default="en_US")
    tags = fields.ListField(fields.EmbeddedDocumentField(TagDocument))

    # relations
    volume_id = fields.StringField(required=True)

    # audit properties
    created_at = fields.DateTimeField(default=datetime.utcnow, required=True)
    created_by = fields.StringField(default="system", required=True)
    updated_at = fields.DateTimeField(default=datetime.utcnow, required=True)
    updated_by = fields.StringField(default="system", required=True)
    deleted_at = fields.DateTimeField(null=True)
    deleted_by = fields.StringField(default="system", required=True)



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from datetime import datetime
from mongoengine import Document, fields
from sweetrpg_catalog_objects.db.embedded.tag.document import TagDocument
from sweetrpg_catalog_objects.db.embedded.property.document import PropertyDocument


class StudioDocument(Document):
    """A mapping object to convert MongoDB data to a Studio object."""

    meta = {
        "indexes": [
            {"name": "studio_name", "fields": ["name"]},
        ],
        "db_alias": "default",
        "collection": "studios",
        "strict": False,
    }

    source_id = fields.StringField()

    # basic properties
    name = fields.StringField(min_length=1, max_length=200, required=True)
    website = fields.StringField()
    notes = fields.StringField()
    tags = fields.ListField(fields.EmbeddedDocumentField(TagDocument))
    properties = fields.ListField(fields.EmbeddedDocumentField(PropertyDocument))

    # relations

    # audit properties
    created_at = fields.DateTimeField(default=datetime.utcnow, required=True)
    created_by = fields.StringField(default="system", required=True)
    updated_at = fields.DateTimeField(default=datetime.utcnow, required=True)
    updated_by = fields.StringField(default="system", required=True)
    deleted_at = fields.DateTimeField(null=True)
    deleted_by = fields.StringField(default="system", required=True)



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from marshmallow_jsonapi.flask import Schema, Relationship
from marshmallow_jsonapi import fields
from sweetrpg_catalog_objects.model.review import Review
from sweetrpg_api_core.schema.base import BaseAPISchema


class ReviewAPISchema(BaseAPISchema):
    model_class = Review

    class Meta:
        type_ = "review"
        self_view = "review_detail"
        self_view_kwargs = {"id": "<id>"}
        self_view_many = "review_list"

    title = fields.String(required=True)  # , load_only=True)
    body = fields.String(required=True)  # , load_only=True)
    language = fields.String(required=True, default="en_US")
    tags = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    volume = Relationship(
        attribute="volume",
        self_view="review_volume",
        self_view_kwargs={"id": "<id>"},
        related_view="volume_detail",
        related_view_kwargs={"volume_id": "<id>"},
        schema="VolumeAPISchema",
        type_="volume",
    )



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
Schema for Volume data.
"""

from marshmallow import fields
from sweetrpg_catalog_objects.model.volume import Volume
from sweetrpg_model_core.schema.base import BaseSchema


class VolumeSchema(BaseSchema):
    model_class = Volume

    title = fields.String(required=True)  # , load_only=True)
    description = fields.String(required=True)  # , load_only=True)
    tags = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    properties = fields.List(fields.Dict(keys=fields.String(required=True), values=fields.String()))
    system_ids = fields.List(fields.String(required=True))  # , load_only=True)
    publisher_ids = fields.List(fields.String(required=True))
    studio_ids = fields.List(fields.String(required=True))
    license_ids = fields.List(fields.String(required=True))



# -*- coding: utf-8 -*-
__author__ = "Paul Schifferer <dm@sweetrpg.com>"
"""
"""

from mongoengine import EmbeddedDocument, fields


class PropertyDocument(EmbeddedDocument):
    """A mapping object for properties."""

    # basic properties
    name = fields.StringField(min_length=1, max_length=50, required=True)
    value = fields.StringField(min_length=2, max_length=200, required=True)
    kind = fields.StringField(min_length=1, max_length=20, required=True)
